@@ -10,63 +10,43 @@ import numpy as np
 from scipy.optimize import curve_fit
 from skimage.color import rgb2gray
 import codecs
+import pickle
 
 class BeamSpot:
     """
     Class to analyse the pictures comming from the MCP
-    
-    Attributes
-    ----------
-    self.fname: string
-        file name of the picture
-    self.img: 2D array
-        picture as an array
-    self.pix: 
-        the pixels along the x axis
-    self.piy:
-        the pixels along the y axis
-    self.Ix: array of floats
-        integral along the x axis
-    self.Iy: array of floats
-        integral along the y axis
-    self.Ax: float
-        Amplitude, fit along the x axis
-    self.Ay: float
-        Amplitude, fit along the y axis
-    self.sigx: float
-        Sigma, fit along the x axis
-    self.sigy: float
-        Sigma, fit along the x axis
-    self.r0x: float
-        Center, fit along the x axis
-    self.r0y: float
-        Center, fit along the x axis
-    self.offsetx: float
-        offset, fit along the x axis
-    self.offsety: float
-        offset, fit along the x axis
-    self.poptx: array of floats
-        The parameters of the fit along the x-axis
-    self.perrx: array of floats
-        Errors on the parameters of the fit along the x-axis
-    self.popty: array of floats
-        The parameters of the fit along the y-axis
-    self.perry: array of floats
-        Errors on the parameters of the fit along the y-axis
-    self.reshape: array of int
-        The parameters to reshape, see help(import_image)
+    #### Attributes
+    * BeamSpot.fname: string, file name of the picture
+    * BeamSpot.img: 2D array, picture as an array
+    * BeamSpot.pix: the pixels along the x axis
+    * BeamSpot.piy: the pixels along the y axis
+    * BeamSpot.Ix: array of floats, integral along the x axis
+    * BeamSpot.Iy: array of floatt, integral along the y axis
+    * BeamSpot.Ax: float, Amplitude, fit along the x axis
+    * BeamSpot.Ay: float, Amplitude, fit along the y axis
+    * BeamSpot.sigx: float, Sigma, fit along the x axis
+    * BeamSpot.sigy: float, Sigma, fit along the x axis
+    * BeamSpot.r0x: float, Center, fit along the x axis
+    * BeamSpot.r0y: float, Center, fit along the x axis
+    * BeamSpot.offsetx: float, offset, fit along the x axis
+    * BeamSpot.offsety: float, offset, fit along the x axis
+    * BeamSpot.poptx: array of floats, the parameters of the fit along the x-axis
+    * BeamSpot.perrx: array of floats, errors on the parameters of the fit along the x-axis
+    * BeamSpot.popty: array of floats, the parameters of the fit along the y-axis
+    * BeamSpot.perry: array of floats, errors on the parameters of the fit along the y-axis
+    * BeamSpot.reshape: array of int, the parameters to reshape, see help(import_image)
     """
     
+    
     def __init__(self,fname,reshape=[]):
         """
         Constructor of the class
-        
-        Parameters
-        ----------
-        fname: string
-            file name of the picture
-        reshape: array of 3 integers (optional)
-            to reshape the pictures (square): x,y,length
+        * Parameters
+            * fname: string, file name of the picture, the accepted file format ["tif","jpg","jpeg","png","asc","bmp"]
+            * reshape: array of 3 integers (optional), to reshape the pictures (square): x,y,length
+        * Example
+            import GBARpy.MCPPicture as mcp
+            bs = mcp.BeamSpot("name.tif")
         """
         self.fname = fname
         self.reshape = reshape
@@ -78,9 +58,18 @@ class BeamSpot:
         self.Ax,self.r0x,self.sigx,self.offsetx = self.poptx
         self.Ay,self.r0y,self.sigy,self.offsety = self.popty
 
+
     def __repr__(self):
         """
         To represent the object as a string
+        * Returns
+            * a string variable
+        * Example
+            import GBARpy.MCPPicture as mcp
+            bs = mcp.BeamSpot("name.tif")
+            repr = bs.__repr__()
+            #or to print it in the python console
+            print(bs)
         """
         res = "Original picture: "+self.fname+'\n\n'
         res += "Integral along the x-axis\n"
@@ -99,6 +88,12 @@ class BeamSpot:
     def plot_Y_int(self,label=""):
         """
         To plot the integral of the picture along the "y" axis
+        * Parameters
+            * label: (optional) a string
+        * Example
+            import GBARpy.MCPPicture as mcp
+            bs = mcp.BeamSpot("name.tif")
+            bs.plot_Y_int("Integral along the y-axis")
         """
         popt = self.popty
         if np.any(np.isnan(popt)):
@@ -115,6 +110,12 @@ class BeamSpot:
     def plot_X_int(self,label=""):
         """
         To plot the integral of the picture along the "x" axis
+        * Parameters
+            * label: (optional) a string
+        * Example
+            import GBARpy.MCPPicture as mcp
+            bs = mcp.BeamSpot("name.tif")
+            bs.plot_X_int("Integral along the x-axis")
         """
         popt = self.poptx
         if np.any(np.isnan(popt)):
@@ -127,17 +128,14 @@ class BeamSpot:
         if len(label) != 0:
             plt.legend()
         
+        
     def plot_X_int_revert(self):
         """
-        To plot the integral of the picture "i" along the "x" axis
-        and reverse the picture
-        
-        Parameters
-        ----------
-        i: int
-            the index of the picture
-        center: boolean (optional)
-            to only show the pixel on the middle of the spot
+        To plot the integral of the picture along the "x" axis and reverse the picture
+        * Example
+            import GBARpy.MCPPicture as mcp
+            bs = mcp.BeamSpot("name.tif")
+            bs.plot_X_int("Integral along the x-axis")
         """
         pix, Ix = self.pix,self.Ix
         popt = self.poptx
@@ -150,14 +148,27 @@ class BeamSpot:
             plt.plot(G,pix,color=p[0].get_color())
         plt.ylim([np.max(pix),0])
     
-    def plot(self,fname="",title="",figsize=(12,10),fontsize=12,ftsizeticks=12):
+    
+    def plot(self,fname="",figsize=(12,10),fontsize=12,ftsizeticks=12):
         """
         To plot the picture and the analysis
+        * Parameters
+            * fname: string (optional), the name of the file to save the plot
+            * figsize: tuple (size in inch X, Y) (optional), size of the figure
+            * fontsize: int (optional), size of the font
+            * ftsizeticks: int (optional), size of the ticks' font
+        * Returns
+            * fig: a matplotlib.pyplot.figure
+        * Example
+            import GBARpy.MCPPicture as mcp
+            bs = mcp.BeamSpot("name.tif")
+            fig = bs.plot("analysis.pdf")
+            # or
+            fig = bs.plot()
+            fig.savefig("analysis.pdf")
         """
-        
         if len(fname) == 0:
-            fname = self.fname+"-analaysis.pdf"
-        
+            fname = None
         fig = plt.figure(figsize=figsize)
         
         plt.subplot(221)
@@ -169,7 +180,7 @@ class BeamSpot:
         plt.title(r"$A_x = $"+str(np.around(self.Ax,1))+" $\sigma_x = $"+str(np.around(self.sigx,1))+" $r_{0x} = $"+str(np.around(self.r0x,1)),fontsize=fontsize,loc='left')
         if not(np.isnan(self.poptx[0])):
             plt.xlim([0,self.Ax/self.sigx/np.sqrt(2*np.pi)])
-        
+
         plt.subplot(222)
         plt.imshow(self.img,vmin=self.img.min(),vmax=self.img.max())
         plt.xticks(fontsize=ftsizeticks)
@@ -185,42 +196,38 @@ class BeamSpot:
         if not(np.isnan(self.popty[0])):
             plt.ylim([0,self.Ay/self.sigy/np.sqrt(2*np.pi)])
         plt.tight_layout()
-        fig.savefig(fname)
-        return fig,fname
-        
+        if fname != None:
+            fig.savefig(fname)
+        return fig
+    
+    
 def normal_distribution(x,s0,x0):
     """
     Normal distribution 
     f(x) = 1/sqrt(2pi)/sigma * exp(-1/2 {(x-mu)/sigma}^2)
-    
-    Parameters
-    ----------
-        x:  an np array
-        s0: floating number, the standard deviation
-        x0: floating number, the mean value, center of the distribution
-    Returns
-    -------
+    * Parameters
+        * x:  an np array
+        * s0: floating number, the standard deviation
+        * x0: floating number, the mean value, center of the distribution
+    * Returns
         the value of the distribution
     """
     return 1/np.sqrt(2*np.pi)/s0*np.exp(-(((x-x0)/s0)**2)/2)
 
 
-
 def gaussian_offset(x,a,x0,s0,c):
     """
     Gaussian distribution with an offset
     f(x) = amplitude/sqrt(2pi)/sigma * exp(-1/2 {(x-mu)/sigma}^2) + offset
     
-    Parameters
-    ----------
-        x:  an np array
-        a:  the amplitude
-        s0: floating number, the standard deviation
-        x0: floating number, the mean value, center of the distribution
-        c:  the offset
-    Returns
-    -------
-        the value of the distribution
+    * Parameters
+        * x:  an np array
+        * a:  the amplitude
+        * s0: floating number, the standard deviation
+        * x0: floating number, the mean value, center of the distribution
+        * c:  the offset
+    * Returns
+        * the value of the distribution
     """
     return a*normal_distribution(x,s0,x0) + c
 
@@ -228,15 +235,11 @@ def gaussian_offset(x,a,x0,s0,c):
 def fit_gaussian_offset_filtered(x,y):
     """
     Fit with the function gaussian_offset.
-    
-    Parameters
-    ----------
-        x: numpy array
-        y: numpy array
-    Returns
-    -------
-        popt,perr: 
-            the parameters and the error of the fit
+    * Parameters
+        * x: numpy array
+        * y: numpy array
+    * Returns
+        * popt,perr: numpy arrays, the parameters and the error of the fit
     """
     x = np.array(x)
     y = np.array(y)
@@ -270,23 +273,19 @@ def fit_gaussian_offset_filtered(x,y):
         perr = [np.nan,np.nan,np.nan,np.nan]
     return popt,perr
 
-
+###################################################################################
 
 def reshapeIMG(img,ix,iy,l):
     """
     To reshape an image to a squared one
-    
-    Parameters
-    ----------
-        img     np.array([np.array]) the image array
-        ix      the index of the center of the square
-        iy      the index of the center of the square
-        l       the half length of the square
-    Returns
-    -------
-    The reshaped picture as an array
+    * Parameters
+        * img: np.array([np.array]) the image array
+        * ix: the index of the center of the square
+        * iy: the index of the center of the square
+        * l: the half length of the square
+    *Returns
+        * The reshaped picture as an array
     """
-    
     ix = int(ix)
     iy = int(iy)
     IX = np.arange(ix-l,ix+l)
@@ -296,19 +295,14 @@ def reshapeIMG(img,ix,iy,l):
     return img
 
 
-
 def import_image(fname,reshape=[]):
     """
     To import the picture as an array
-    
-    Parameters
-    ----------
-        fname   (string) name of the file
-        reshape (array) to reshape the picture as a square
-                [ix,iy,l]
-    Returns
-    -------
-        the picture as an array
+    * Parameters
+        * fname: string, name of the file
+        * reshape: array, to reshape the picture as a square [ix,iy,l]
+    * Returns
+        * the picture as 2D array
     """
     fileformat = ["tif","jpg","jpeg","png","asc","bmp"]
     ext = fname.split('.')[len(fname.split('.'))-1]
@@ -332,20 +326,13 @@ def import_image(fname,reshape=[]):
     return img
 
 
-
 def integrate_picture_along_Y(img):
     """
     To integrate the picture along th Y axis
-    
-    Parameters
-    ----------
-        img: 
-            image as a 2D numpy array
-    Returns
-    -------
-        (pix,Iy):
-            A tupple with pix the pixel numbers as a numpy array
-            and Iy the integral as a numpy array
+    * Parameters
+        * img: image as a 2D numpy array
+    * Returns
+        * (pix,Iy): A tupple with pix the pixel numbers as a numpy array and Iy the integral as a numpy array
     """
     I = np.zeros_like(img[0])
     for l in img:
@@ -355,20 +342,13 @@ def integrate_picture_along_Y(img):
     return pix,Iy
 
 
-
 def integrate_picture_along_X(img):
     """
     To integrate the picture along th X axis
-    
-    Parameters
-    ----------
-        img: 
-            image as a 2D numpy array
-    Returns
-    -------
-        (pix,Ix):
-            A tupple with pix the pixel numbers as a numpy array
-            and Ix the integral as a numpy array
+    *Parameters
+        * img: image as a 2D numpy array
+    * Returns
+        * (pix,Ix): A tupple with pix the pixel numbers as a numpy array and Ix the integral as a numpy array
     """
     pix,ix = integrate_picture_along_Y(np.transpose(img))
     Ix = np.array([])
@@ -377,14 +357,11 @@ def integrate_picture_along_X(img):
     return pix,Ix
 
 
-
 def getIndexStr(N,i):
     """
     To convert an int 'i' to a string
-    
-    Example
-    -------
-    getIndexStr(100,15) returns '015'
+    * Example
+        getIndexStr(100,15) #returns '015'
     """
     if i < 0 or i > N:
         raise ValueError("N >= i or i > 0 is required")
@@ -396,4 +373,118 @@ def getIndexStr(N,i):
 
 
 
-            
\ No newline at end of file
+class MCPParams:
+    """
+    Class to store the parameters of the MCP to adapt the analysis
+    
+    #### Attributes
+    * MCPParams.R: radius of the mirror in mm
+    * MCPParams.x0: x position of the center of the mirror in pixels
+    * MCPParams.y0: y position of the center of the mirror in pixels
+    * MCPParams.R0: radius of the mirror in pixels
+    * MCPParams.ratio: mm/pixels ratio
+    * MCPParams.canBePlot: boolean to know if the mirror can be plot
+    * MCPParams.ratioIsSet: boolean to know if a ratio has been defined
+    """
+    def __init__(self,name=None,R=None,x0=None,y0=None,
+                R0=None,ratio=None):
+            """
+            Constructor of the class
+            * Parameters
+                * name: (optional) string, the name of the MCP
+                * R: (optional), float, radius of the mirror in mm
+                * x0: (optional), int, x position of the center of the mirror in pixels
+                * y0: (optional), int, y position of the center of the mirror in pixels
+                * R0: (optional), int, radius of the mirror in pixels
+                * ratio: (optional), mm/pixels ratio
+                    if R and R0 have been defined, then ratio is defined automatically
+            """
+            self.name = name
+            self.R = R
+            self.x0 = x0
+            self.y0 = y0
+            self.R0 = R0
+            self.ratio = ratio
+            if R!= None and R0!=None:
+                self.ratio = R/R0
+            self.checkRatioIsSet()
+    
+    
+    def __repr__(self):
+        """
+        To translate the object as a string
+        * Returns
+            * the string
+        * Example
+            params = MCPParams()
+            print(params)
+            #or
+            r = params.__repr__()
+        """
+        res = "name: "+self.name+"\n"
+        res+= "R: "+str(self.R)+"\n"
+        res+= "x0: "+str(self.x0)+"\n"
+        res+= "y0: "+str(self.y0)+"\n"
+        res+= "R0: "+str(self.R0)+"\n"
+        res+= "ratio: "+str(self.ratio)+"\n"
+        return res
+    
+    def defineRatio(self,mm,pix):
+        """
+        To define the ratio mm vs pixels
+        * Parameters
+            mm: float, a distance in mm
+            pix: float, the equivalent distance in pixels
+        """
+        try:
+            r = mm/pix
+            self.ratio = r
+            self.checkRatioIsSet()
+        except:
+            print("The setting of the ratio has failed")
+            
+            
+    def checkRatioIsSet(self):
+        """
+        To check is the ratio has been set
+        * Return
+            boolean
+        """
+        if self.ratio == None:
+            self.ratioIsSet = False
+        else:
+            self.ratioIsSet = True
+        return self.ratioIsSet
+        
+        
+    def save_conf(self,fname):
+        """
+        To save the parameters of the MCP as a binary file
+        Use .mcp extension
+        * Parameters
+            fname: string, the name of the binary file
+        * Example
+            params = MCPParams()
+            params.save_config("config.mcp")
+        """
+        with open(fname, 'wb') as f1:
+            pickle.dump(self, f1)
+        print("Parameters saved as",fname)
+
+############################################################################
+def import_config(fname):
+    """
+    To import the MCP parameters from a binary file
+    * Parameters
+        fname: string, the file's name
+    """
+    with open(fname, 'rb') as f1:
+        pp = None
+        try:
+            pp = pickle.load(f1)
+            print("MCP parameters",pp.name,"from",fname,"imported")
+        except:
+            print("Importation of",fname,"has failed")
+        return pp
+
+            
